package com.qhtx.eta.common.aspect;

import com.alibaba.fastjson.JSON;
import com.google.common.util.concurrent.RateLimiter;
import com.qhtx.eta.common.annotation.DataOperator;
import com.qhtx.eta.common.config.DataSourceProperties;
import com.qhtx.eta.common.eunms.ETAApiType;
import com.qhtx.eta.common.eunms.ErrorEnum;
import com.qhtx.eta.common.eunms.OperatorEnum;
import com.qhtx.eta.common.exception.ETAException;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.reflect.MethodSignature;
import org.springframework.stereotype.Component;

import javax.annotation.Resource;
import java.beans.Introspector;
import java.beans.PropertyDescriptor;
import java.sql.SQLRecoverableException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

@Aspect
@Component
@Slf4j
public class DataOperatorAspect {

    @Resource(name = "dataRateLimiter")
    private RateLimiter dataRateLimiter;

    @Resource(name = "quotaRateLimiter")
    private RateLimiter quotaRateLimiter;

    @Resource
    private DataSourceProperties dataSourceProperties;

    @Around("@annotation(com.qhtx.eta.common.annotation.DataOperator)")
    public Object handleDataOperator(ProceedingJoinPoint joinPoint) throws Throwable {
        Object[] parameters = joinPoint.getArgs();
        MethodSignature signature = (MethodSignature) joinPoint.getSignature();
        DataOperator dataOperator = signature.getMethod().getAnnotation(DataOperator.class);
        Map<String, Object> operatorParams = new HashMap<>();
        if (dataOperator.operator() == OperatorEnum.BATCH) {
            if (parameters.length > 0 && parameters[0] instanceof List) {
                List<?> dataList = (List<?>) parameters[0];
                Class<?> clazz = dataOperator.dataType();
                if (!dataList.isEmpty() && clazz.isAssignableFrom(dataList.get(0).getClass())) {
                    PropertyDescriptor[] propertyDescriptors = Introspector.getBeanInfo(clazz).getPropertyDescriptors();
                    for (String field : dataOperator.fields()) {
                        for (PropertyDescriptor pd : propertyDescriptors) {
                            if (field.equals(pd.getName())) {
                                operatorParams.put(pd.getName(), pd.getReadMethod().invoke(dataList.get(0)));
                                break;
                            }
                        }
                    }
                }
            }
        } else {
            Class<?> clazz = dataOperator.dataType();
            if (parameters.length > 0 && clazz.isAssignableFrom(parameters[0].getClass())) {
                PropertyDescriptor[] propertyDescriptors = Introspector.getBeanInfo(clazz).getPropertyDescriptors();
                for (String field : dataOperator.fields()) {
                    for (PropertyDescriptor pd : propertyDescriptors) {
                        if (field.equals(pd.getName())) {
                            operatorParams.put(pd.getName(), pd.getReadMethod().invoke((parameters[0])));
                            break;
                        }
                    }
                }
            }
        }
        if (!limit(dataOperator.apiType())) {
            if (dataOperator.apiType() == ETAApiType.QUOTA) {
                throw new ETAException(ErrorEnum.SYSTEM_ERROR.getCode(), String.format(ErrorEnum.SYSTEM_ERROR.getMsg(), "数据库操作请求被拒绝,当前无可用请求令牌"));
            }
            if (dataOperator.apiType() == ETAApiType.DATA) {
                throw new ETAException(ErrorEnum.SYSTEM_ERROR.getCode(), String.format(ErrorEnum.SYSTEM_ERROR.getMsg(), "数据库操作请求被拒绝,当前无可用请求令牌,请求指标{" + JSON.toJSONString(operatorParams) + "}"));
            }
        }

        int retryCount = 0;
        try {
            // 执行方法
            return joinPoint.proceed();
        } catch (Exception e) {
            SQLRecoverableException sqlException = findSQLRecoverableCause(e);
            if (sqlException != null) {
                log.warn(String.format(ErrorEnum.DATASOURCE_EXECUTE_ERROR.getMsg(), "可恢复的数据库异常"));
                while (retryCount < dataSourceProperties.getRetryTimes()) {
                    retryCount++;
                    try {
                        TimeUnit.MILLISECONDS.sleep(dataSourceProperties.getRetryInterval());
                        log.info("尝试重新提交数据库执行操作：{}次", retryCount);
                        Object obj = joinPoint.proceed();
                        if (obj != null) {
                            log.info("重试成功：{}，第{}次尝试", JSON.toJSONString(operatorParams), retryCount);
                        }
                        return obj;
                    } catch (InterruptedException interruptEx) {
                        Thread.currentThread().interrupt(); // 保持中断状态
                        log.error(ErrorEnum.INTERNAL_SERVER_ERROR.getMsg(), interruptEx);
                        throw new ETAException(ErrorEnum.INTERNAL_SERVER_ERROR);
                    } catch (SQLRecoverableException retrySqlEx) {
                        // 如果重试也失败了，并且还是SQLRecoverableException，则继续循环或在达到重试次数后抛出
                        sqlException = retrySqlEx;
                    }
                }
                throw new ETAException(ErrorEnum.DATASOURCE_EXECUTE_ERROR.getCode(), String.format(ErrorEnum.DATASOURCE_EXECUTE_ERROR.getMsg(), sqlException.getMessage()));
            } else {
                // 对其他类型的异常进行处理
                throw new ETAException(ErrorEnum.SYSTEM_ERROR.getCode(), e.getMessage());
            }
        }
    }

    // 方法来查找并返回SQLRecoverableException，如果存在的话
    private SQLRecoverableException findSQLRecoverableCause(Throwable throwable) {
        if (throwable instanceof SQLRecoverableException) {
            return (SQLRecoverableException) throwable;
        } else if (throwable.getCause() != null) {
            return findSQLRecoverableCause(throwable.getCause());
        }
        return null;
    }

    private boolean limit(ETAApiType etAApiType) {
        switch (etAApiType) {
            case DATA:
                return dataRateLimiter.tryAcquire();
            case QUOTA:
                return quotaRateLimiter.tryAcquire();
            default:
                return true;
        }
    }


}
